@@ -8,11 +8,13 @@ from napari import Viewer
 from pandas import DataFrame
 from qtpy.QtWidgets import QTableWidget, QTableWidgetItem, QWidget, QGridLayout, QPushButton, QFileDialog
 from skimage.measure import regionprops_table
+from napari_tools_menu import register_function
 
 @napari_hook_implementation
 def napari_experimental_provide_function():
     return [duplicate_current_frame]
 
+@register_function(menu="Utilities > Duplicate current timepoint")
 def duplicate_current_frame(image: ImageData, napari_viewer : Viewer, axis : int = 0) -> ImageData:
     current_dim_value = napari_viewer.dims.current_step[axis]
     return np.take(image, current_dim_value, axis)
